@@ -13,41 +13,19 @@
 ##############################################################################
 """Security proxy tests
 """
+import io
+import os
 import unittest
 
-from zope.security._compat import PYTHON2, PURE_PYTHON
+from zope.security._compat import PURE_PYTHON
 
 
-def _skip_if_not_Py2(testfunc):
-    return unittest.skipUnless(PYTHON2, "Only on Py2")(testfunc)
-
 # pylint:disable=protected-access,eval-used,too-many-lines,too-many-public-methods
 
-
-if not PYTHON2:  # pragma: PY3
-    def coerce(*args):
-        raise NotImplementedError("Not on Python 3")
-    cmp = coerce
-    long = int
-    unicode = str
-
-
-class AbstractProxyTestBase(object):
+class AbstractProxyTestBase:
 
     # pylint:disable=no-member,blacklisted-name
 
-    assertRaisesRegex = getattr(unittest.TestCase, 'assertRaisesRegex',
-                                unittest.TestCase.assertRaisesRegexp)
-
-    # The names of attributes that are spelled different on Py2
-    # vs Py3
-    itruediv = '__itruediv__' if not PYTHON2 else '__idiv__'
-    idiv = itruediv
-    div = '__truediv__' if not PYTHON2 else '__div__'
-    truediv = div
-    getslice = '__getitem__' if not PYTHON2 else '__getslice__'
-    setslice = '__setitem__' if not PYTHON2 else '__setslice__'
-
     def _getTargetClass(self):
         raise NotImplementedError("Subclass responsibility")
 
@@ -58,7 +36,7 @@ class AbstractProxyTestBase(object):
         self.assertRaises(ValueError, self._makeOne, object(), None)
 
     def test___getattr___w_checker_ok(self):
-        class Foo(object):
+        class Foo:
             bar = 'Bar'
         target = Foo()
         checker = DummyChecker()
@@ -71,7 +49,7 @@ class AbstractProxyTestBase(object):
     def test___getattr___w_checker_unauthorized(self):
         from zope.security.interfaces import Unauthorized
 
-        class Foo(object):
+        class Foo:
             bar = 'Bar'
         target = Foo()
         checker = DummyChecker(Unauthorized)
@@ -82,7 +60,7 @@ class AbstractProxyTestBase(object):
     def test___getattr___w_checker_forbidden_attribute(self):
         from zope.security.interfaces import ForbiddenAttribute
 
-        class Foo(object):
+        class Foo:
             bar = 'Bar'
         target = Foo()
         checker = DummyChecker(ForbiddenAttribute)
@@ -93,7 +71,7 @@ class AbstractProxyTestBase(object):
         self.assertEqual(checker._checked, 'bar')
 
     def test__getattr__w_checker_ok_dynamic_attribute_called_once(self):
-        class Dynamic(object):
+        class Dynamic:
             count = 0
 
             def __getattr__(self, name):
@@ -112,7 +90,7 @@ class AbstractProxyTestBase(object):
         self.assertEqual(1, target.count)
 
     def test___setattr___w_checker_ok(self):
-        class Foo(object):
+        class Foo:
             bar = 'Bar'
         target = Foo()
         checker = DummyChecker()
@@ -125,7 +103,7 @@ class AbstractProxyTestBase(object):
     def test___setattr___w_checker_unauthorized(self):
         from zope.security.interfaces import Unauthorized
 
-        class Foo(object):
+        class Foo:
             bar = 'Bar'
         target = Foo()
         checker = DummyChecker(Unauthorized)
@@ -136,7 +114,7 @@ class AbstractProxyTestBase(object):
     def test___setattr___w_checker_forbidden_attribute(self):
         from zope.security.interfaces import ForbiddenAttribute
 
-        class Foo(object):
+        class Foo:
             bar = 'Bar'
         target = Foo()
         checker = DummyChecker(ForbiddenAttribute)
@@ -145,7 +123,7 @@ class AbstractProxyTestBase(object):
         self.assertEqual(checker._checked, 'bar')
 
     def test___delattr___w_checker_ok(self):
-        class Foo(object):
+        class Foo:
             bar = None
         target = Foo()
         target.bar = 'Bar'
@@ -159,7 +137,7 @@ class AbstractProxyTestBase(object):
     def test___delattr___w_checker_unauthorized(self):
         from zope.security.interfaces import Unauthorized
 
-        class Foo(object):
+        class Foo:
             def __init__(self):
                 self.bar = 'Bar'
         target = Foo()
@@ -172,7 +150,7 @@ class AbstractProxyTestBase(object):
     def test___delattr___w_checker_forbidden_attribute(self):
         from zope.security.interfaces import ForbiddenAttribute
 
-        class Foo(object):
+        class Foo:
             def __init__(self):
                 self.bar = 'Bar'
         target = Foo()
@@ -190,20 +168,19 @@ class AbstractProxyTestBase(object):
 
     def test___str___checker_forbids_str(self):
         from zope.security.interfaces import ForbiddenAttribute
-        from zope.security._compat import _BUILTINS
         from zope.security.proxy import _fmt_address
         target = object()
         checker = DummyChecker(ForbiddenAttribute)
         proxy = self._makeOne(target, checker)
         address = _fmt_address(target)
         self.assertEqual(str(proxy),
-                         '<security proxied %s.object '
-                         'instance at %s>' % (_BUILTINS, address))
+                         '<security proxied builtins.object '
+                         'instance at %s>' % address)
 
     def test___str___fails_return(self):
         from zope.security.interfaces import ForbiddenAttribute
 
-        class CustomStr(object):
+        class CustomStr:
             def __str__(self):
                 "<CustomStr>"  # Docstring, not a return
 
@@ -223,20 +200,19 @@ class AbstractProxyTestBase(object):
 
     def test___repr___checker_forbids_str(self):
         from zope.security.interfaces import ForbiddenAttribute
-        from zope.security._compat import _BUILTINS
         from zope.security.proxy import _fmt_address
         target = object()
         checker = DummyChecker(ForbiddenAttribute)
         proxy = self._makeOne(target, checker)
         address = _fmt_address(target)
         self.assertEqual(repr(proxy),
-                         '<security proxied %s.object '
-                         'instance at %s>' % (_BUILTINS, address))
+                         '<security proxied builtins.object '
+                         'instance at %s>' % address)
 
     def test___str___falls_through_to_repr_when_both_allowed(self):
         from zope.security.interfaces import ForbiddenAttribute
 
-        class CustomRepr(object):
+        class CustomRepr:
             def __repr__(self):
                 return "<CustomRepr>"
 
@@ -251,7 +227,7 @@ class AbstractProxyTestBase(object):
     def test___str___doesnot_fall_through_to_repr_when_str_not_allowed(self):
         from zope.security.interfaces import ForbiddenAttribute
 
-        class CustomRepr(object):
+        class CustomRepr:
             def __repr__(self):
                 return "<CustomRepr>"
 
@@ -265,7 +241,7 @@ class AbstractProxyTestBase(object):
     def test___str___doesnot_fall_through_to_repr_when_repr_not_allowed(self):
         from zope.security.interfaces import ForbiddenAttribute
 
-        class CustomRepr(object):
+        class CustomRepr:
             def __repr__(self):
                 return "<CustomRepr>"
 
@@ -279,7 +255,7 @@ class AbstractProxyTestBase(object):
     def test___str___falls_through_to_repr_but_repr_fails_return(self):
         from zope.security.interfaces import ForbiddenAttribute
 
-        class CustomRepr(object):
+        class CustomRepr:
             def __repr__(self):
                 "<CustomRepr>"  # Docstring, not a return
 
@@ -291,36 +267,6 @@ class AbstractProxyTestBase(object):
         with self.assertRaises(TypeError):
             repr(proxy)
 
-    @_skip_if_not_Py2
-    def test___cmp___w_self(self):  # pragma: no cover
-        target = object()
-        checker = object()  # checker not consulted
-        proxy = self._makeOne(target, checker)
-        self.assertEqual(cmp(proxy, proxy), 0)
-
-    @_skip_if_not_Py2
-    def test___cmp___w_target(self):  # pragma: no cover
-        target = object()
-        checker = object()  # checker not consulted
-        proxy = self._makeOne(target, checker)
-        self.assertEqual(cmp(proxy, target), 0)
-
-    @_skip_if_not_Py2
-    def test___cmp___w_other(self):  # pragma: no cover
-        target = object()
-        other = object()
-        checker = object()  # checker not consulted
-        proxy = self._makeOne(target, checker)
-        self.assertNotEqual(cmp(proxy, other), 0)
-
-    @_skip_if_not_Py2
-    def test___cmp___w_other_proxy(self):  # pragma: no cover
-        target = object()
-        checker = object()  # checker not consulted
-        proxy = self._makeOne(target, checker)
-        o_proxy = self._makeOne(target, checker)
-        self.assertEqual(cmp(proxy, o_proxy), 0)
-
     def test__le__(self):
         target = 1
         checker = object()  # checker not consulted
@@ -352,7 +298,7 @@ class AbstractProxyTestBase(object):
         self.assertEqual(hash(proxy), hash(target))
 
     def test___call___w_checker_ok(self):
-        class Foo(object):
+        class Foo:
             def __call__(self):
                 return 'Bar'
         target = Foo()
@@ -365,7 +311,7 @@ class AbstractProxyTestBase(object):
     def test___call___w_checker_unauthorized(self):
         from zope.security.interfaces import Unauthorized
 
-        class Foo(object):
+        class Foo:
             def __call__(self):
                 raise AssertionError("Never called")
         target = Foo()
@@ -377,7 +323,7 @@ class AbstractProxyTestBase(object):
     def test___call___w_checker_forbidden_attribute(self):
         from zope.security.interfaces import ForbiddenAttribute
 
-        class Foo(object):
+        class Foo:
             def __call__(self):
                 raise AssertionError("Never called")
         target = Foo()
@@ -416,57 +362,6 @@ class AbstractProxyTestBase(object):
         self.assertRaises(ForbiddenAttribute, float, proxy)
         self.assertEqual(checker._checked, '__float__')
 
-    @_skip_if_not_Py2
-    def test___long___w_checker_allows(self):  # pragma: no cover
-        target = 3.0
-        checker = DummyChecker()
-        proxy = self._makeOne(target, checker)
-        self.assertEqual(long(proxy), long(target))
-        self.assertEqual(checker._checked, '__long__')
-
-    @_skip_if_not_Py2
-    def test___long___w_checker_forbids(self):  # pragma: no cover
-        from zope.security.interfaces import ForbiddenAttribute
-        target = 3
-        checker = DummyChecker(ForbiddenAttribute)
-        proxy = self._makeOne(target, checker)
-        self.assertRaises(ForbiddenAttribute, long, proxy)
-        self.assertEqual(checker._checked, '__long__')
-
-    @_skip_if_not_Py2
-    def test___oct___w_checker_allows(self):  # pragma: no cover
-        target = 3
-        checker = DummyChecker()
-        proxy = self._makeOne(target, checker)
-        self.assertEqual(oct(proxy), oct(target))
-        self.assertEqual(checker._checked, '__oct__')
-
-    @_skip_if_not_Py2
-    def test___oct___w_checker_forbids(self):  # pragma: no cover
-        from zope.security.interfaces import ForbiddenAttribute
-        target = 3
-        checker = DummyChecker(ForbiddenAttribute)
-        proxy = self._makeOne(target, checker)
-        self.assertRaises(ForbiddenAttribute, oct, proxy)
-        self.assertEqual(checker._checked, '__oct__')
-
-    @_skip_if_not_Py2
-    def test___hex___w_checker_allows(self):  # pragma: no cover
-        target = 3
-        checker = DummyChecker()
-        proxy = self._makeOne(target, checker)
-        self.assertEqual(hex(proxy), hex(target))
-        self.assertEqual(checker._checked, '__hex__')
-
-    @_skip_if_not_Py2
-    def test___hex___w_checker_forbids(self):  # pragma: no cover
-        from zope.security.interfaces import ForbiddenAttribute
-        target = 3
-        checker = DummyChecker(ForbiddenAttribute)
-        proxy = self._makeOne(target, checker)
-        self.assertRaises(ForbiddenAttribute, hex, proxy)
-        self.assertEqual(checker._checked, '__hex__')
-
     def test___add___w_checker_allows(self):
         target = 3
         checker = DummyChecker()
@@ -512,27 +407,12 @@ class AbstractProxyTestBase(object):
         self.assertRaises(ForbiddenAttribute, lambda: proxy * 2)
         self.assertEqual(checker._checked, '__mul__')
 
-    def test___div___w_checker_allows(self):
-        target = 3
-        checker = DummyChecker()
-        proxy = self._makeOne(target, checker)
-        self.assertEqual(proxy / 2, target / 2)
-        self.assertEqual(checker._checked, self.div)
-
-    def test___div___w_checker_forbids(self):
-        from zope.security.interfaces import ForbiddenAttribute
-        target = 3
-        checker = DummyChecker(ForbiddenAttribute)
-        proxy = self._makeOne(target, checker)
-        self.assertRaises(ForbiddenAttribute, lambda: proxy / 2)
-        self.assertEqual(checker._checked, self.div)
-
     def test___truediv___w_checker_allows(self):
         target = 3.0
         checker = DummyChecker()
         proxy = self._makeOne(target, checker)
         self.assertEqual(proxy / 2, target / 2)
-        self.assertEqual(checker._checked, self.truediv)
+        self.assertEqual(checker._checked, '__truediv__')
 
     def test___truediv___w_checker_forbids(self):
         from zope.security.interfaces import ForbiddenAttribute
@@ -540,7 +420,7 @@ class AbstractProxyTestBase(object):
         checker = DummyChecker(ForbiddenAttribute)
         proxy = self._makeOne(target, checker)
         self.assertRaises(ForbiddenAttribute, lambda: proxy / 2)
-        self.assertEqual(checker._checked, self.truediv)
+        self.assertEqual(checker._checked, '__truediv__')
 
     def test___floordiv___w_checker_allows(self):
         target = 3
@@ -758,23 +638,6 @@ class AbstractProxyTestBase(object):
         self.assertRaises(ForbiddenAttribute, lambda: proxy | 2)
         self.assertEqual(checker._checked, '__or__')
 
-    @_skip_if_not_Py2
-    def test___coerce___w_checker_allows(self):  # pragma: no cover
-        target = 3
-        checker = DummyChecker()
-        proxy = self._makeOne(target, checker)
-        self.assertEqual(coerce(proxy, 4.0), coerce(target, 4.0))
-        self.assertEqual(checker._checked, '__coerce__')
-
-    @_skip_if_not_Py2
-    def test___coerce___w_checker_forbids(self):  # pragma: no cover
-        from zope.security.interfaces import ForbiddenAttribute
-        target = 3
-        checker = DummyChecker(ForbiddenAttribute)
-        proxy = self._makeOne(target, checker)
-        self.assertRaises(ForbiddenAttribute, coerce, proxy, 4.0)
-        self.assertEqual(checker._checked, '__coerce__')
-
     def test___iadd___not_inplace_checker_allows(self):
         target = 3
         checker = DummyChecker()
@@ -785,7 +648,7 @@ class AbstractProxyTestBase(object):
         self.assertEqual(checker._checked, '__iadd__')
 
     def test___iadd___inplace_checker_allows(self):
-        class Foo(object):
+        class Foo:
             def __init__(self, value):
                 self.value = value
 
@@ -819,7 +682,7 @@ class AbstractProxyTestBase(object):
         self.assertEqual(checker._checked, '__isub__')
 
     def test___isub___inplace_checker_allows(self):
-        class Foo(object):
+        class Foo:
             def __init__(self, value):
                 self.value = value
 
@@ -853,7 +716,7 @@ class AbstractProxyTestBase(object):
         self.assertEqual(checker._checked, '__imul__')
 
     def test___imul___inplace_checker_allows(self):
-        class Foo(object):
+        class Foo:
             def __init__(self, value):
                 self.value = value
 
@@ -877,41 +740,6 @@ class AbstractProxyTestBase(object):
             proxy *= 3
         self.assertEqual(checker._checked, '__imul__')
 
-    def test___idiv___not_inplace_checker_allows(self):
-        target = 6
-        checker = DummyChecker()
-        proxy = before = self._makeOne(target, checker)
-        proxy /= 3
-        self.assertIsNot(proxy, before)
-        self.assertEqual(proxy, 2)
-        self.assertEqual(checker._checked, self.idiv)
-
-    def test___idiv___inplace_checker_allows(self):
-        class Foo(object):
-            def __init__(self, value):
-                self.value = value
-
-            def __idiv__(self, rhs):
-                self.value /= rhs
-                return self
-            __itruediv__ = __idiv__
-        target = Foo(6)
-        checker = DummyChecker()
-        proxy = before = self._makeOne(target, checker)
-        proxy /= 3
-        self.assertIs(proxy, before)
-        self.assertEqual(target.value, 2)
-        self.assertEqual(checker._checked, self.idiv)
-
-    def test___idiv___w_checker_forbids(self):
-        from zope.security.interfaces import ForbiddenAttribute
-        target = 6
-        checker = DummyChecker(ForbiddenAttribute)
-        proxy = self._makeOne(target, checker)
-        with self.assertRaises(ForbiddenAttribute):
-            proxy /= 3
-        self.assertEqual(checker._checked, self.idiv)
-
     def test___itruediv___not_inplace_checker_allows(self):
         target = 6
         checker = DummyChecker()
@@ -919,24 +747,23 @@ class AbstractProxyTestBase(object):
         proxy /= 3
         self.assertIsNot(proxy, before)
         self.assertEqual(proxy, 2)
-        self.assertEqual(checker._checked, self.itruediv)
+        self.assertEqual(checker._checked, '__itruediv__')
 
     def test___itruediv___inplace_checker_allows(self):
-        class Foo(object):
+        class Foo:
             def __init__(self, value):
                 self.value = value
 
             def __itruediv__(self, rhs):
                 self.value /= rhs
                 return self
-            __idiv__ = __itruediv__
         target = Foo(6)
         checker = DummyChecker()
         proxy = before = self._makeOne(target, checker)
         proxy /= 3
         self.assertIs(proxy, before)
         self.assertEqual(target.value, 2)
-        self.assertEqual(checker._checked, self.itruediv)
+        self.assertEqual(checker._checked, '__itruediv__')
 
     def test___itruediv___w_checker_forbids(self):
         from zope.security.interfaces import ForbiddenAttribute
@@ -945,7 +772,7 @@ class AbstractProxyTestBase(object):
         proxy = self._makeOne(target, checker)
         with self.assertRaises(ForbiddenAttribute):
             proxy /= 3
-        self.assertEqual(checker._checked, self.itruediv)
+        self.assertEqual(checker._checked, '__itruediv__')
 
     def test___ifloordiv___not_inplace_checker_allows(self):
         target = 6
@@ -957,7 +784,7 @@ class AbstractProxyTestBase(object):
         self.assertEqual(checker._checked, '__ifloordiv__')
 
     def test___ifloordiv___inplace_checker_allows(self):
-        class Foo(object):
+        class Foo:
             def __init__(self, value):
                 self.value = value
 
@@ -991,7 +818,7 @@ class AbstractProxyTestBase(object):
         self.assertEqual(checker._checked, '__imod__')
 
     def test___imod___inplace_checker_allows(self):
-        class Foo(object):
+        class Foo:
             def __init__(self, value):
                 self.value = value
 
@@ -1025,7 +852,7 @@ class AbstractProxyTestBase(object):
         self.assertEqual(checker._checked, '__ipow__')
 
     def test___ipow___inplace_checker_allows(self):
-        class Foo(object):
+        class Foo:
             def __init__(self, value):
                 self.value = value
 
@@ -1059,7 +886,7 @@ class AbstractProxyTestBase(object):
         self.assertEqual(checker._checked, '__ilshift__')
 
     def test___ilshift___inplace_checker_allows(self):
-        class Foo(object):
+        class Foo:
             def __init__(self, value):
                 self.value = value
 
@@ -1093,7 +920,7 @@ class AbstractProxyTestBase(object):
         self.assertEqual(checker._checked, '__irshift__')
 
     def test___irshift___inplace_checker_allows(self):
-        class Foo(object):
+        class Foo:
             def __init__(self, value):
                 self.value = value
 
@@ -1127,7 +954,7 @@ class AbstractProxyTestBase(object):
         self.assertEqual(checker._checked, '__iand__')
 
     def test___iand___inplace_checker_allows(self):
-        class Foo(object):
+        class Foo:
             def __init__(self, value):
                 self.value = value
 
@@ -1161,7 +988,7 @@ class AbstractProxyTestBase(object):
         self.assertEqual(proxy, 4)
 
     def test___ixor___inplace_checker_allows(self):
-        class Foo(object):
+        class Foo:
             def __init__(self, value):
                 self.value = value
 
@@ -1197,7 +1024,7 @@ class AbstractProxyTestBase(object):
         self.assertEqual(checker._checked, '__ior__')
 
     def test___ior___inplace_checker_allows(self):
-        class Foo(object):
+        class Foo:
             def __init__(self, value):
                 self.value = value
 
@@ -1323,38 +1150,11 @@ class AbstractProxyTestBase(object):
         self.assertRaises(ForbiddenAttribute, _try)
         self.assertEqual(checker._checked, '__setitem__')
 
-    @_skip_if_not_Py2
-    def test___getslice___w_checker_allows(self):  # pragma: no cover
-        target = [0, 1, 2]
-        checker = DummyChecker()
-        proxy = self._makeOne(target, checker)
-        self.assertEqual(proxy[1:3], [1, 2])
-        self.assertEqual(checker._checked, self.getslice)
-
     def test___getslice___error_propagates(self):
         class Missing(Exception):
             pass
 
-        class Get(object):
-            def __getitem__(self, x):
-                raise Missing('__getitem__')
-
-            def __getslice__(self, start, stop):  # PY2
-                raise Missing("__getslice__")  # pragma: no cover
-        target = Get()
-        checker = DummyChecker()
-        proxy = self._makeOne(target, checker)
-        with self.assertRaisesRegex(Missing,
-                                    self.getslice):
-            proxy[1:2]
-
-        self.assertEqual(checker._checked, self.getslice)
-
-    def test___getslice___dne_uses_getitem(self):
-        class Missing(Exception):
-            pass
-
-        class Get(object):
+        class Get:
             def __getitem__(self, x):
                 raise Missing('__getitem__')
 
@@ -1365,62 +1165,30 @@ class AbstractProxyTestBase(object):
                                     '__getitem__'):
             proxy[1:2]
 
-        self.assertEqual(checker._checked, self.getslice)
+        self.assertEqual(checker._checked, '__getitem__')
 
-    @_skip_if_not_Py2
-    def test___getslice___w_checker_forbids(self):  # pragma: no cover
-        from zope.security.interfaces import ForbiddenAttribute
-        target = [0, 1, 2]
-        checker = DummyChecker(ForbiddenAttribute)
-        proxy = self._makeOne(target, checker)
-        self.assertRaises(ForbiddenAttribute, lambda: proxy[0:2])
-        self.assertEqual(checker._checked, '__getslice__')
+    def test___getslice___dne_uses_getitem(self):
+        class Missing(Exception):
+            pass
 
-    @_skip_if_not_Py2
-    def test___setslice___w_checker_allows(self):  # pragma: no cover
-        target = [0, 1, 2]
+        class Get:
+            def __getitem__(self, x):
+                raise Missing('__getitem__')
+
+        target = Get()
         checker = DummyChecker()
         proxy = self._makeOne(target, checker)
-        proxy[1:3] = [3, 4]
-        self.assertEqual(target, [0, 3, 4])
-        self.assertEqual(checker._checked, '__setslice__')
+        with self.assertRaisesRegex(Missing,
+                                    '__getitem__'):
+            proxy[1:2]
 
-    @_skip_if_not_Py2
-    def test___setslice___w_checker_forbids(self):  # pragma: no cover
-        from zope.security.interfaces import ForbiddenAttribute
-        target = [0, 1, 2]
-        checker = DummyChecker(ForbiddenAttribute)
-        proxy = self._makeOne(target, checker)
-
-        def _try():
-            proxy[1:3] = [3, 4]
-        self.assertRaises(ForbiddenAttribute, _try)
-        self.assertEqual(checker._checked, '__setslice__')
+        self.assertEqual(checker._checked, '__getitem__')
 
     def test___setslice___error_propagates(self):
         class Missing(Exception):
             pass
 
-        class Set(object):
-            def __setitem__(self, k, v):
-                raise Missing('__setitem__')
-
-            def __setslice__(self, start, stop, value):  # PY2
-                raise Missing("__setslice__")  # pragma: no cover
-        target = Set()
-        checker = DummyChecker()
-        proxy = self._makeOne(target, checker)
-        with self.assertRaisesRegex(Missing,
-                                    self.setslice):
-            proxy[1:2] = 1
-
-        self.assertEqual(checker._checked, self.setslice)
-
-    def test___setslice___dne_uses_setitem(self):
-        class Missing(Exception):
-            pass
-
-        class Set(object):
+        class Set:
             def __setitem__(self, k, v):
                 raise Missing('__setitem__')
 
@@ -1431,7 +1199,24 @@ class AbstractProxyTestBase(object):
                                     '__setitem__'):
             proxy[1:2] = 1
 
-        self.assertEqual(checker._checked, self.setslice)
+        self.assertEqual(checker._checked, '__setitem__')
+
+    def test___setslice___dne_uses_setitem(self):
+        class Missing(Exception):
+            pass
+
+        class Set:
+            def __setitem__(self, k, v):
+                raise Missing('__setitem__')
+
+        target = Set()
+        checker = DummyChecker()
+        proxy = self._makeOne(target, checker)
+        with self.assertRaisesRegex(Missing,
+                                    '__setitem__'):
+            proxy[1:2] = 1
+
+        self.assertEqual(checker._checked, '__setitem__')
 
     def test___getitem___mapping_hit_w_checker_allows(self):
         target = {'a': 0, 'b': 1, 'c': 2}
@@ -1490,60 +1275,10 @@ class AbstractProxyTestBase(object):
                         z = eval(expr)
                         first = 0
                     else:
-                        self.assertEqual(removeSecurityProxy(eval(expr)), z,
-                                         "x=%r; y=%r; expr=%r" % (x, y, expr))
-
-    @_skip_if_not_Py2
-    def test___unicode___allowed_by_default(self):  # pragma: no cover
-        # https://github.com/zopefoundation/zope.security/issues/10
-        class Foo(object):
-            def __unicode__(self):
-                return u'I am unicode'
-
-        checker = object()  # checker not consulted
-        target = Foo()
-        proxy = self._makeOne(target, checker)
-        self.assertEqual(unicode(target), u'I am unicode')
-        self.assertEqual(unicode(target), unicode(proxy))
-
-    @_skip_if_not_Py2
-    def test___unicode___falls_through_to_str_by_default(self):  # pragma: no cover # noqa: E501 line too long
-        # https://github.com/zopefoundation/zope.security/issues/10
-        class Foo(object):
-            def __str__(self):
-                return 'I am str'
-
-        checker = object()  # checker not consulted
-        target = Foo()
-        proxy = self._makeOne(target, checker)
-        self.assertEqual(unicode(target), u'I am str')
-        self.assertIsInstance(unicode(target), unicode)
-        self.assertEqual(unicode(target), unicode(proxy))
-        self.assertIsInstance(unicode(proxy), unicode)
-
-    @_skip_if_not_Py2
-    def test___unicode___falls_through_to_str_even_if_str_not_allowed(self):  # pragma: no cover # noqa: E501 line too long
-        # https://github.com/zopefoundation/zope.security/issues/10
-        # Note that this is inconsistent with str() and probably not a good
-        # idea overall, so this test is strictly a regression test.
-        from zope.security.interfaces import ForbiddenAttribute
-
-        class Foo(object):
-            def __str__(self):
-                return 'I am str'
-
-        target = Foo()
-        checker = DummyChecker(ForbiddenAttribute)
-        proxy = self._makeOne(target, checker)
-        self.assertEqual(unicode(target), u'I am str')
-        self.assertIsInstance(unicode(target), unicode)
-
-        # Asking for the unicode of the proxy silently falls through
-        # to the str without any checks
-        self.assertEqual(unicode(target), unicode(proxy))
-
-        # And set str itself is checked and proxied
-        self.assertIn("<security proxied", str(proxy))
+                        self.assertEqual(
+                            removeSecurityProxy(eval(expr)),
+                            z,
+                            "x={!r}; y={!r}; expr={!r}".format(x, y, expr))
 
 
 @unittest.skipIf(PURE_PYTHON,
@@ -1583,7 +1318,9 @@ class ProxyPyTests(AbstractProxyTestBase,
         self.assertIs(checker, sub.get_checker())
 
     def test_ctor_w_checker(self):
-        from zope.security.proxy import getObjectPy, getCheckerPy
+        from zope.security.proxy import getCheckerPy
+        from zope.security.proxy import getObjectPy
+
         # Can't access '_wrapped' / '_checker' in C version
         target = object()
         checker = object()
@@ -1656,7 +1393,7 @@ class ProxyPyTests(AbstractProxyTestBase,
         self.assertIs(self, getObjectPy(self))
 
     def test_get_reduce(self):
-        class Reduce(object):
+        class Reduce:
             def __reduce__(self):
                 return 1
 
@@ -1669,7 +1406,7 @@ class ProxyPyTests(AbstractProxyTestBase,
         self.assertEqual(2, proxy.__reduce_ex__(2))
 
     def test__module__(self):
-        class WithModule(object):
+        class WithModule:
             __module__ = 'foo'
 
         module = WithModule()
@@ -1677,7 +1414,7 @@ class ProxyPyTests(AbstractProxyTestBase,
         self.assertEqual(WithModule.__module__, proxy.__module__)
 
 
-class DummyChecker(object):
+class DummyChecker:
     _proxied = _checked = None
 
     def __init__(self, raising=None, allowed=()):
@@ -1728,7 +1465,7 @@ class Test_isinstance(unittest.TestCase):
         return isinstance(object, cls)
 
     def test_w_unproxied_object(self):
-        class Foo(object):
+        class Foo:
             pass
         target = Foo()
         self.assertTrue(self._callFUT(target, Foo))
@@ -1738,7 +1475,7 @@ class Test_isinstance(unittest.TestCase):
         from zope.security.checker import Checker
         from zope.security.proxy import Proxy
 
-        class Foo(object):
+        class Foo:
             pass
         target = Foo()
         proxy = Proxy(target, Checker({}))
@@ -1748,14 +1485,16 @@ class Test_isinstance(unittest.TestCase):
 
 # pre-geddon
 
-class Checker(object):
+class Checker:
 
     ok = 1
 
     unproxied_types = {str, }
 
     def check_getattr(self, _object, name):
-        if name not in ("foo", "next", "__class__", "__name__", "__module__"):
+        if name in ("__class__", "__name__", "__module__"):
+            return
+        if not self.ok or name not in ("__next__", "foo"):
             raise RuntimeError
 
     def check_setattr(self, _object, name):
@@ -1800,21 +1539,10 @@ class Something:
 
     def __next__(self):
         return 42  # Infinite sequence
-    next = __next__
 
     def __len__(self):
         return 42
 
-    def __nonzero__(self):  # PY2
-        return 1  # pragma: no cover
-
-    def __getslice__(self, i, j):  # PY2
-        return [42]  # pragma: no cover
-
-    def __setslice__(self, i, j, value):  # PY2
-        if value != [42]:  # pragma: no cover
-            raise AssertionError("Value should be [42]")
-
     def __contains__(self, x):
         return x == 42
 
@@ -1926,17 +1654,10 @@ class ProxyFactoryTests(unittest.TestCase):
         self.shouldFail(iter, self.p)
 
     def testNextOK(self):
-        self.assertEqual(self.p.next(), 42)
+        self.assertEqual(next(self.p), 42)
 
     def testNextFail(self):
-        self.shouldFail(self.p.next)
-
-    @_skip_if_not_Py2
-    def testCompareOK(self):  # pragma: no cover
-        self.assertEqual(cmp(self.p, self.x), 0)
-
-#    def testCompareFail(self):
-#        self.shouldFail(cmp, self.p, self.x)
+        self.shouldFail(next, self.p)
 
     def testHashOK(self):
         self.assertEqual(hash(self.p), hash(self.x))
@@ -1956,11 +1677,6 @@ class ProxyFactoryTests(unittest.TestCase):
     def testLenFail(self):
         self.shouldFail(len, self.p)
 
-    @_skip_if_not_Py2
-    def testSliceOK(self):  # pragma: no cover
-        from zope.security.proxy import removeSecurityProxy
-        self.assertEqual(removeSecurityProxy(self.p[:]), [42])
-
     def testSliceFail(self):
         self.shouldFail(lambda: self.p[:])
 
@@ -1989,7 +1705,7 @@ class ProxyFactoryTests(unittest.TestCase):
     def testProxiedClassicClassAsDictKey(self):
         from zope.security.proxy import ProxyFactory
 
-        class C(object):
+        class C:
             pass
         d = {C: C()}
         pC = ProxyFactory(C, self.c)
@@ -1998,15 +1714,13 @@ class ProxyFactoryTests(unittest.TestCase):
     def testProxiedNewClassAsDictKey(self):
         from zope.security.proxy import ProxyFactory
 
-        class C(object):
+        class C:
             pass
         d = {C: C()}
         pC = ProxyFactory(C, self.c)
         self.assertEqual(d[pC], d[C])
 
     unops = ["-x", "+x", "abs(x)", "~x", "int(x)", "float(x)"]
-    if PYTHON2:
-        unops.append("long(x)")  # pragma: no cover
 
     def _make_eval(self, expr, locs):
         def _eval(*args):
@@ -2021,8 +1735,6 @@ class ProxyFactoryTests(unittest.TestCase):
         from zope.security.proxy import ProxyFactory
         from zope.security.proxy import removeSecurityProxy
         self.c.unproxied_types = {str, int, float}
-        if PYTHON2:
-            self.c.unproxied_types.add(long)  # pragma: no cover
 
         for expr in self.unops:
             x = 1
@@ -2031,20 +1743,9 @@ class ProxyFactoryTests(unittest.TestCase):
             x = ProxyFactory(1, self.c)
             z = eval(expr)
             self.assertEqual(removeSecurityProxy(z), y,
-                             "x=%r; expr=%r" % (x, expr))
+                             "x={!r}; expr={!r}".format(x, expr))
             self.shouldFail(self._make_eval(expr, locals()), x)
 
-    @_skip_if_not_Py2
-    def test_odd_unops(self):  # pragma: no cover
-        # unops that don't return a proxy
-        P = self.c.proxy
-        for func in (
-                hex, oct,
-                # lambda x: not x,
-        ):
-            self.assertEqual(func(P(100)), func(100))
-            self.shouldFail(func, P(100))
-
     binops = [
         "x+y", "x-y", "x*y", "x/y", "divmod(x, y)", "x**y", "x//y",
         "x<<y", "x>>y", "x&y", "x|y", "x^y",
@@ -2061,8 +1762,10 @@ class ProxyFactoryTests(unittest.TestCase):
                         z = eval(expr)
                         first = 0
                     else:
-                        self.assertEqual(removeSecurityProxy(eval(expr)), z,
-                                         "x=%r; y=%r; expr=%r" % (x, y, expr))
+                        self.assertEqual(
+                            removeSecurityProxy(eval(expr)),
+                            z,
+                            "x={!r}; y={!r}; expr={!r}".format(x, y, expr))
                         self.shouldFail(self._make_eval(expr, locals()), x, y)
 
     def test_inplace(self):
@@ -2094,73 +1797,11 @@ class ProxyFactoryTests(unittest.TestCase):
             pa **= 2
         self.shouldFail(doit2)
 
-    @_skip_if_not_Py2
-    def test_coerce(self):  # pragma: no cover
-        from zope.security.proxy import removeSecurityProxy
-        P = self.c.proxy
-
-        x = P(1)
-        y = P(2)
-        a, b = coerce(x, y)
-        self.assertIs(a, x)
-        self.assertIs(b, y)
-
-        x = P(1)
-        y = P(2.1)
-        a, b = coerce(x, y)
-        self.assertEqual(removeSecurityProxy(a), 1.0)
-        self.assertIs(b, y)
-        self.assertIs(type(removeSecurityProxy(a)), float)
-        self.assertIs(b, y)
-
-        x = P(1.1)
-        y = P(2)
-        a, b = coerce(x, y)
-        self.assertIs(a, x)
-        self.assertEqual(removeSecurityProxy(b), 2.0)
-        self.assertIs(a, x)
-        self.assertIs(type(removeSecurityProxy(b)), float)
-
-        x = P(1)
-        y = 2
-        a, b = coerce(x, y)
-        self.assertIs(a, x)
-        self.assertIs(b, y)
-
-        x = P(1)
-        y = 2.1
-        a, b = coerce(x, y)
-        self.assertIs(type(removeSecurityProxy(a)), float)
-        self.assertIs(b, y)
-
-        x = P(1.1)
-        y = 2
-        a, b = coerce(x, y)
-        self.assertIs(a, x)
-        self.assertIs(type(removeSecurityProxy(b)), float)
-
-        x = 1
-        y = P(2)
-        a, b = coerce(x, y)
-        self.assertIs(a, x)
-        self.assertIs(b, y)
-
-        x = 1.1
-        y = P(2)
-        a, b = coerce(x, y)
-        self.assertIs(a, x)
-        self.assertIs(type(removeSecurityProxy(b)), float)
-
-        x = 1
-        y = P(2.1)
-        a, b = coerce(x, y)
-        self.assertIs(type(removeSecurityProxy(a)), float)
-        self.assertIs(b, y)
-
     def test_iterate_interface(self):
         # This used to work on Python 2, but fail on Python 3.
         # See https://github.com/zopefoundation/zope.interface/issues/141
         from zope.interface import Interface
+
         from zope.security.proxy import ProxyFactory
 
         class IFoo(Interface):
@@ -2171,6 +1812,16 @@ class ProxyFactoryTests(unittest.TestCase):
         self.assertEqual(list(IFoo), ['x'])
         self.assertEqual(list(proxy), list(IFoo))
 
+    def test_method_wrapper(self):
+        from zope.security.proxy import ProxyFactory
+
+        self.assertEqual(ProxyFactory({}).__repr__(), '{}')
+
+    def test_builtin_method(self):
+        from zope.security.proxy import ProxyFactory
+
+        self.assertEqual(ProxyFactory(io.FileIO(os.devnull, 'rb').read)(), b'')
+
 
 def test_using_mapping_slots_hack():
     """The security proxy will use mapping slots, on the checker to go faster
@@ -2248,12 +1899,11 @@ class LocationProxySecurityCheckerTests(unittest.TestCase):
         # Regression test for a problem introduced in 3.8.1 and fixed in
         # 3.8.3. For details see change log.
         import sys
+        from importlib import reload as _reload
+
         from zope.location.location import LocationProxy
+
         import zope.security
-        try:
-            from importlib import reload as _reload
-        except ImportError:  # pragma: PY2
-            _reload = reload  # noqa: F821 undefined name 'reload'
 
         # This attribute is set when zope.security.decorator is imported, to
         # show that it will be set too, if zope.security.proxy is imported
